precision highp float;
attribute vec2 a_position;
attribute vec2 a_position2;
attribute vec4 a_texCoord;
attribute vec4 a_hitTestColour;
attribute vec2 a_textureBank;
attribute vec4 a_colour;
attribute float a_circleRadius;
attribute float a_width;
attribute float a_startAngle;
attribute float a_endAngle;
attribute vec3 a_shadowColour;
attribute float a_type;

uniform vec2 u_resolution;
uniform mediump float u_hitTest;
uniform mat4 u_transform;
//uniform float u_zoom;

varying float v_width;
varying float v_circleRadius;
varying vec4 v_texCoord;
varying vec4 v_colour;
varying vec4 v_hitTestColour;
varying vec2 v_textureBank;
varying float v_pixel;
varying vec2 v_donutSegment;
varying float v_type;
varying float v_aaDistance;


float u_zoom = 1.0;

vec2 line(float density, float type) {
// Divide minWidth by pixel density so retina screen retain correct width
  float minWidth = 1.0 / density;
  int lineType = int(type);
  // disable aa when type == 1
  bool isSharpLine = lineType == 1;
  bool isDashedLine = lineType == 2;
  bool isDottedLine = lineType == 3;
  // Make it wider to allow the anti aliasing to eat into the line in order to make it smooth
  float aaWidthBuffer = 2.0;
  // don't widen for dots
  v_aaDistance = 0.25;
  if(isSharpLine) {
    //ensure aa never kicks in while avoiding an if statement in fragment shader
    v_aaDistance = 1.0;
  }
  
  if(u_hitTest == 1.0) {
    minWidth = 4.0;
  }
  // increase min-width for dotted line #2426
  minWidth = max(minWidth, step(2.5, a_texCoord.w) * 2.0);
  
  vec2 position;
  
  // Direction of the normal vector
  float normalPointsUpward = a_texCoord.y - 1.0;
  
  // The UV coords from 0 -> 1, used for dashing, the line number is added
  // for the case when lines are joined
  float uv = a_texCoord.x;
  
  vec2 distanceVector = a_position2 - a_position;
  
  vec2 normal = normalize(vec2(-distanceVector.y, distanceVector.x));
  
  // Whether the vertex is at the start or end of the segment.
  if(int(a_texCoord.x) == 0) {
   position = a_position;
  } else {
   position = a_position2;
  }
  float finalWidth = ceil(a_width * aaWidthBuffer);
  //if(int(u_viewIndependent) == 0) {
   finalWidth = max((finalWidth) * u_zoom, minWidth * aaWidthBuffer) / u_zoom;
  //}
  // get vector for line
  vec2 normalVec = normal * (finalWidth/2.0) * normalPointsUpward;
  position = position + normalVec;
  
 
  
  return position;
}

void main() {
    vec2 position = a_position;

    if(a_type <= 1.5) {
      position = line(1.0, 1.0);
    }
   // convert the rectangle from pixels to 0.0 to 1.0
   vec2 zeroToOne = position / u_resolution.xy;
   // convert from 0->1 to 0->2
   vec2 zeroToTwo = zeroToOne * 2.0;

   // convert from 0->2 to -1->+1 (clipspace)
   vec2 clipSpace = zeroToTwo - 1.0;

   gl_Position = u_transform * vec4(clipSpace * vec2(1, -1), 0, 1);

   // pass the texCoord to the fragment shader
   // The GPU will interpolate this value between points.
   v_texCoord = a_texCoord;
  
   v_colour = a_colour;
   v_hitTestColour = a_hitTestColour;
   v_textureBank = a_textureBank;
   v_circleRadius = a_circleRadius;
   v_width = a_width;
   // the size of a pixel in screenspace clip coordinates
   v_pixel = 1.0 / (a_circleRadius * 2.0) / u_zoom;
   v_donutSegment = vec2(a_startAngle, a_endAngle);
   v_type = a_type;
   vec3 shadCol = a_shadowColour;
}
